@@ -1,4 +1,4 @@
-'''
+"""
 Genetic Programming for Quant
 
 Author: UePG
@@ -6,8 +6,8 @@ Author: UePG
 Reference:
 [1] https://gplearn.readthedocs.io/en/stable/
 [2] J. Koza, "Genetic Programming", 1992.
-'''
+"""
 
-__version__ = '0.1.4'
+__version__ = "0.1.5"
 
-__all__ = ['Function', 'Fitness', 'Backtester', 'SymbolicRegressor']
\ No newline at end of file
+__all__ = ["Function", "Fitness", "Backtester", "SymbolicRegressor"]
